"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing_extensions import Annotated, NotRequired, TypedDict


class FiservInstallmentOptionsTypedDict(TypedDict):
    number_of_installments: NotRequired[Nullable[int]]
    r"""Passes the `order.installmentOptions.numberOfInstallments` field to the Fiserv API."""
    installments_interest: NotRequired[Nullable[bool]]
    r"""Passes the `order.installmentOptions.installmentsInterest` field to the Fiserv API."""
    installment_delay_months: NotRequired[Nullable[int]]
    r"""Passes the `order.installmentOptions.installmentDelayMonths` field to the Fiserv API."""
    merchant_advice_code_supported: NotRequired[Nullable[bool]]
    r"""Passes the `order.installmentOptions.merchantAdviceCodeSupported` field to the Fiserv API."""


class FiservInstallmentOptions(BaseModel):
    number_of_installments: Annotated[
        OptionalNullable[int], pydantic.Field(alias="numberOfInstallments")
    ] = UNSET
    r"""Passes the `order.installmentOptions.numberOfInstallments` field to the Fiserv API."""

    installments_interest: Annotated[
        OptionalNullable[bool], pydantic.Field(alias="installmentsInterest")
    ] = UNSET
    r"""Passes the `order.installmentOptions.installmentsInterest` field to the Fiserv API."""

    installment_delay_months: Annotated[
        OptionalNullable[int], pydantic.Field(alias="installmentDelayMonths")
    ] = UNSET
    r"""Passes the `order.installmentOptions.installmentDelayMonths` field to the Fiserv API."""

    merchant_advice_code_supported: Annotated[
        OptionalNullable[bool], pydantic.Field(alias="merchantAdviceCodeSupported")
    ] = UNSET
    r"""Passes the `order.installmentOptions.merchantAdviceCodeSupported` field to the Fiserv API."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "numberOfInstallments",
            "installmentsInterest",
            "installmentDelayMonths",
            "merchantAdviceCodeSupported",
        ]
        nullable_fields = [
            "numberOfInstallments",
            "installmentsInterest",
            "installmentDelayMonths",
            "merchantAdviceCodeSupported",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
